/****************************************************************************
*
*            COPYRIGHT 1990,91,92 BY GRACILIS INC.
*
*			 623 Palace St.
*			 Aurora, Il. 60506
*
*			 (708)-801-8800		Office
*			 (708)-844-0183		(FAX - Support BBS)
*
* GRACILIS, INC., MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS 
* SOFTWARE FOR ANY PURPOSE.  
*
* THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
* 
* Permission is granted for non-commercial use/distribution only, as long as
* this copyright header is included intact and unaltered.
*
******************************************************************************/

/* Callbook server */
#include <stdio.h>
#include <string.h>
#include "global.h"
#include "files.h"
#include "mbuf.h"
#include "socket.h"
#include "session.h"
#include "proc.h"
#include "dirutil.h"
#include "commands.h"
#include "mailbox.h"
#include "config.h"
#include "cmdparse.h"
#include "netuser.h"

#include "gcallbk.h"		/* get the port # for the server to watch */

char	*Callserver = NULLCHAR;

static int cdbstart __ARGS((int argc,char *argv[],void *p));
static int cdb0 __ARGS((int argc,char *argv[],void *p));
static void calldbd __ARGS((int s,void *unused,void *p));
static int docallserver __ARGS((int argc,char *argv[],void *p));
static int dodbname __ARGS((int argc,char *argv[],void *p));
static int dodblookup __ARGS((int argc,char *argv[],void *p));
static int dodbstats __ARGS((int argc,char *argv[],void *p));
static int dodbtype __ARGS((int argc,char *argv[],void *p));


static struct cmds Call_cmds[] = {
	"database",		dodbname,		0, 		0,	NULLCHAR,
	"lookup",		dodblookup,		1024, 		0,	NULLCHAR,
	"server",		docallserver,	0, 		0,	NULLCHAR,
	"startserver",	cdbstart,		256,	0,	NULLCHAR,
	"status",		dodbstats,		0,		0,	NULLCHAR,
	"stopserver",	cdb0,			0,		0,	NULLCHAR,
	"type",			dodbtype,		0, 		0,	NULLCHAR,
	NULLCHAR,
} ;


static int Scallbook = -1;	/* Prototype socket for service */

static int32 request_cnt = 0;	/* keep track of the number of requests */

char	*CDBversion = "2.0";				/* Integer release number for the server */

char	*CDBname;					/* pointer to complete file spec for */
									/* callsign database... This is built */
									/* at the time of starting the server */
									/* OR by a user command */

int		CDBtype = BUCKMASTER;		/* Default to a BUCKMASTER CD-ROM */
									/* The other accepted value is "FCC" */

/************************************************************************/
/* These are just the Default values to build the Database filename... 	*/
/* They may be overridden by user commands 								*/
/* Note that the directory and filenames are correct for the BUCKMASTER */
/* HAMCALL CD-ROM as of 4/92 										    */
/************************************************************************/

static char	*CDBdrv = "F:";			/* Disk drive containing the database */

static char	*CDBpath = "/ham0/";	/* Directory name where database resides */
									/* Always specify full directory path */
									/* including an ending slash "/"      */
									/* NOTE: Use FORWARD SLASH "/"        */

static char	*CDBfile = "hamcall.129";	/* Database file name */



/* Command multiplexer */
int
dogcallbook(argc,argv,p)
int argc ;
char *argv[] ;
void *p;
{
	return subcmd(Call_cmds,argc,argv,p) ;
}


/* Start up callsign database service */
static int
cdbstart(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	struct sockaddr_in lsocket;
	int s;

	if(Scallbook != -1){
		return 0;
	}
	psignal(Curproc,0);	/* Don't keep the parser waiting */
	chname(Curproc,"Callbook listener");

	lsocket.sin_family = AF_INET;
	lsocket.sin_addr.s_addr = INADDR_ANY;
	if(argc < 2)
		lsocket.sin_port = IPPORT_CALLDB;
	else
		lsocket.sin_port = atoi(argv[1]);

	Scallbook = socket(AF_INET,SOCK_STREAM,0);
	bind(Scallbook,(char *)&lsocket,sizeof(lsocket));
	listen(Scallbook,1);


	/* If no database has been specified, use the defaults. */
	if(CDBname == NULLCHAR)
	{
		/* Allocate and set up the default database name */
		CDBname = callocw(1,(strlen(CDBpath) + strlen(CDBdrv) + strlen(CDBfile) + 1));
		strcpy(CDBname,CDBdrv);
		strcat(CDBname,CDBpath);
		strcat(CDBname,CDBfile);
	}


	for(;;){
		if((s = accept(Scallbook,NULLCHAR,(int *)NULL)) == -1)
			break;	/* Service is shutting down */

		/* Spawn a server */
		newproc("calldbd",1024,calldbd,s,NULL,NULL,0);
	}
	return 0;
}


static void
calldbd(s,unused,p)
int s;
void *unused;
void *p;
{
	char user[80];
	FILE *fp;
	char *file,*cp;

	sockmode(s,SOCK_ASCII);
	sockowner(s,Curproc);
	log(s,"open Callbook");
	request_cnt++;			/* bump the attempts stat. */
	recvline(s,user,80);
	rip(user);
	if(strlen(user) == 0)
	{
			usprintf(s,"No Callbook information available\n");
	}
	else 
	{
		cb_lookup(s,user);
	}
	close_s(s);
	log(s,"close Callbook");
}


static int
cdb0(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	close_s(Scallbook);
	Scallbook = -1;

	if(CDBname != NULLCHAR) 
	{
		free(CDBname);
		CDBname = NULLCHAR;
	}
	request_cnt = 0;			/* Zero the request stats */

	return 0;
}

static int
docallserver(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	if(argc >= 2)
	{
		/* free up the old name... */
		if(Callserver != NULLCHAR) free(Callserver);

		if( strcmpi("none",argv[1]) == 0) Callserver = NULLCHAR;

		else
		{
			/* set up a new one... */
			Callserver = strdup(argv[1]);
		}
	}
		
	if(Callserver != NULLCHAR)
		tprintf("The current server for requests from this node is: %s\n\n",Callserver); 

	else tprintf("Callbook request services are disabled.\n\n");

	if(argc < 2)
	{
		tprintf("Usage: callbook server hostname\n");
		tprintf("   or: callbook server ip_address\n");
		tprintf("   or: callbook server None\n");
	}
	return 0;
}

static int
dodbname(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	if(argc >=2)
	{

		/* free up the old name... */
		if(CDBname != NULLCHAR)
		{
			free(CDBname);
			CDBname = NULLCHAR;
		}

		/* set up the new one... */
		CDBname = strdup(argv[1]);
	}
	tprintf("Database file: %s\n",(CDBname != NULLCHAR) ? CDBname : "None" );


	if(argc < 2) tprintf("Usage: callbook database <drive:/path/filename>\n");

	return 0;
}


static int
dodbtype(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	if(argc >=2)
	{

		if(strncmpi("buckmaster",argv[1],strlen(argv[1])) == 0)
		{
			CDBtype = BUCKMASTER;
		}
		else CDBtype = FCCSTYLE;
	}
	tprintf("Database type: %s\n",(CDBtype == BUCKMASTER) 
									? "Buckmaster" : "FCC" );


	if(argc < 2) tprintf("Usage: callbook type <buckmaster | fcc>\n");

	return 0;
}


static int
dodblookup(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	struct sockaddr_in sock;
	char *cp;
	int s,i;
	struct mbuf *bp;
	struct session *sp;

	if(argc < 2)
	{
		tprintf("Usage: callbook lookup <callsign>\n");
		return 0;
	}

	request_cnt++;			/* bump the attempts stat. */

	/* Allocate a session descriptor */

	/* This ridiculous junk is because too many people are */
	/* hacking in too many different directions */
	/* DGL 8/20/92 */

#ifdef GCALLBOOK_GRI	/* PA0GRI */
  	if((sp = newsession(argv[1],FINGER,0,1)) == NULLSESSION)
#endif

#ifdef GCALLBOOK_WG7J	/* WG7J */
  	if((sp = newsession(argv[1],FINGER,0)) == NULLSESSION)
#endif

#ifdef GCALLBOOK_KA9Q 	/* KA9Q */
  	if((sp = newsession(argv[1],FINGER)) == NULLSESSION)
#endif

  	{
		tprintf("Too many sessions\n");
		keywait(NULLCHAR,1);
		return 1;
	}
	if(Callserver == NULLCHAR)
	{
		tprintf("Callbook Services Not Configured.\n");
		keywait(NULLCHAR,1);
		freesession(sp);
		return 0;
	}

	sp->ttystate.echo = sp->ttystate.edit = 0;
	sp->flowmode = 1;
	sock.sin_family = AF_INET;
	sock.sin_port = IPPORT_CALLDB;
	for(i=1;i<argc;i++)
	{
		cp = Callserver;
    	strupr(argv[i]);
    	tprintf("Requesting Callbook Record for \"%s\" ",argv[i]);
		if((sock.sin_addr.s_addr = resolve(cp)) == 0)
		{
			tprintf("Remote Callserver %s unknown\n",cp);
			continue;
		}
		tprintf("from %s",psocket((struct sockaddr *)&sock));
		if((sp->s = s = socket(AF_INET,SOCK_STREAM,0)) == -1)
		{
			tprintf("Can't create socket\n");
			break;
		}
		sockmode(s,SOCK_ASCII);
		if(connect(s,(char *)&sock,sizeof(sock)) == -1)
		{
			cp = sockerr(s);
			tprintf(" -- Connect failed: %s\n",cp != NULLCHAR ? cp : "");
			close_s(s);
			sp->s = -1;
			continue;
		}
    	tprintf("\n");
		usprintf(s,"%s\n",argv[i]);
		usflush(Curproc->output);
		while(recv_mbuf(s,&bp,0,NULLCHAR,(int *)0) > 0)
		{
			send_mbuf(Curproc->output,bp,0,NULLCHAR,0);
		}
		close_s(s);
		sp->s = -1;
	}
	keywait(NULLCHAR,1);
	freesession(sp);
	return 0;
}

static int
dodbstats(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	tprintf("\nCallbook Server Rel %s : %s\n",CDBversion, (Scallbook != -1) ? "Active" : "Disabled");
	tprintf("                Requests: %ld\n",request_cnt);

	if((argc >= 2) && (strcmpi(argv[1],"clear") == 0)) request_cnt = 0;
	return 0;
}
