/* Miscellaneous interger and IP address format conversion subroutines
 * Copyright 1991 Phil Karn, KA9Q
 */
#define LINELEN 256
#include <ctype.h>
#include <stdio.h>
#include "global.h"
#include "netuser.h"
#include "domain.h"
#include "socket.h"

int Net_error;

/* Convert Internet address in ascii dotted-decimal format (44.0.0.1) to
 * binary IP address
 */
int32
aton(s)
register char *s;
{
	int32 n;

	register int i;

	n = 0;
	if(s == NULLCHAR)
		return 0;
	for(i=24;i>=0;i -= 8){
		/* Skip any leading stuff (e.g., spaces, '[') */
		while(*s != '\0' && !isdigit(*s))
			s++;
		if(*s == '\0')
			break;
		n |= (int32)atoi(s) << i;
		if((s = strchr(s,'.')) == NULLCHAR)
			break;
		s++;
	}
	return n;
}
/* Convert an internet address (in host byte order) to a dotted decimal ascii
 * string, e.g., 255.255.255.255\0
 */
char *
inet_ntoa(a)
int32 a;
{
	static char buf[60];
	char * sp;
	unsigned a_lblw;

	/* We output a dotted decimal quad instead of a host name if any
	   of the following are true:
	   1. The last octet of the address is 0 or 255, or
	   2. The "domain translate" flag is set to disabled,
	   3. An attempt to resolve the IP address to a host name failed.
	   (The order of these tests is critical to the operation of the
	   code below.)  -- N1BEE
	*/
	if( (0 == (a_lblw = lobyte(loword(a)))) ||
    	    (a_lblw == 255) ||
	    (DTranslate == FALSE) ||
	    (NULLCHAR == (sp = resolve_a(a,!DVerbose))) ) {
		sprintf(buf,"%u.%u.%u.%u",
			hibyte(hiword(a)),
			lobyte(hiword(a)),
			hibyte(loword(a)),
			a_lblw);
	} else {
		sprintf(buf,"%.59s",sp);
		free(sp);
	}
	return buf;
}
/* Convert hex-ascii string to long integer */
long
htol(s)
char *s;
{
	long ret;
	char c;

	ret = 0;
	while((c = *s++) != '\0'){
		c &= 0x7f;
		if(c == 'x')
			continue;	/* Ignore 'x', e.g., '0x' prefixes */
		if(c >= '0' && c <= '9')
			ret = ret*16 + (c - '0');
		else if(c >= 'a' && c <= 'f')
			ret = ret*16 + (10 + c - 'a');
		else if(c >= 'A' && c <= 'F')
			ret = ret*16 + (10 + c - 'A');
		else
			break;
	}
	return ret;
}

char *
pinet(s)
struct socket *s;
{
	static char buf[80];
	char port[10];

	switch(s->port) {
		case IPPORT_ECHO:		/* Echo data port */
			sprintf(port,"echo");
			break;
		case IPPORT_DISCARD:		/* Discard data port */
			sprintf(port,"discard");
			break;
		case IPPORT_FTPD:		/* FTP Data port */
			sprintf(port,"ftpd");
			break;
		case IPPORT_FTP:		/* FTP Control port */
			sprintf(port,"ftp");
			break;
		case IPPORT_TELNET:		/* Telnet port */
			sprintf(port,"telnet");
			break;
		case IPPORT_SMTP:		/* Mail port */
			sprintf(port,"smtp");
			break;
		case IPPORT_DOMAIN:		/* Domain Nameserver */
			sprintf(port,"domain");
			break;
		case IPPORT_BOOTPS:		/* Domain Nameserver */
			sprintf(port,"bootps");
			break;
		case IPPORT_BOOTPC:		/* Domain Nameserver */
			sprintf(port,"bootpc");
			break;
		case IPPORT_TFTP:		/* TFTP Data port */
			sprintf(port,"tftpd");
			break;
		case IPPORT_FINGER:		/* Finger port */
			sprintf(port,"finger");
			break;
		case IPPORT_TTYLINK:		/* Ttylink port */
			sprintf(port,"ttylink");
			break;
		case IPPORT_POP2:		/* POP2 port */
			sprintf(port,"pop2");
			break;
		case IPPORT_POP3:		/* POP3 port */
			sprintf(port,"pop3");
			break;
		case IPPORT_NNTP:		/* NNTP port */
			sprintf(port,"nntp");
			break;
		case IPPORT_RLOGIN:		/* RLOGIN port */
			sprintf(port,"rlogin");
			break;
		case IPPORT_TIMED:		/* TIMED port */
			sprintf(port,"timed");
			break;
		case IPPORT_LPD:		/* LPD port */
			sprintf(port,"lpd");
			break;
		case IPPORT_RIP:		/* Routing Information Protocol */
			sprintf(port,"rip");
			break;
		case IPPORT_REMOTE:		/* Pulled out of the air */
			sprintf(port,"remote");
			break;
		case IPPORT_CONVERS:
			sprintf(port,"convers");
			break;
		default:
			sprintf(port,"%u",s->port);
			break;
	}

	sprintf(buf,"%s:%s",inet_ntoa(s->address),port);
	return buf;
}

